function ZhUtil(ApplicationContext, Query, $filter, dataSourceHolder, ZhLogger) {
	var self = this;

	this.evaluate = function($string) {
		var $a, $function_detector, $function_new, $ret, $string_ret, _i, _len;
		if (angular.isFunction($string)) {
			$string_ret = $string();
		} else {
			$string_ret = $string;
			$function_detector = /\@(.*?)\@/g;
			if ($string_ret) {
				$ret = $string_ret.toString().match($function_detector);
				if ($ret) {
					var keys = Object.keys(ApplicationContext);
					var args = keys.map(function(key) {
						return ApplicationContext[key];
					});
					var argsToFunction = ApplicationContext.getArgsToFunction();
					for (_i = 0, _len = $ret.length; _i < _len; _i++) {
						$a = $ret[_i];
						$function_new = $a.replace(/\@/g, '');
						/*jslint evil: true */
						eval('$return = (( function(' + argsToFunction.keys.join(' , ') + '){ return ' +
							$function_new +
							';} ).bind(ApplicationContext) ).apply(ApplicationContext, argsToFunction.args);');
						if ($string_ret.length === $a.length) {
							$string_ret = $return;
						} else {
							$string_ret = $string_ret.replace($a, $return);
						}
					}
				}
			}
		}
		return $string_ret;
	};

	this.formatLangToMobiscroll = function(lang) {
		if (lang) {
			lang = lang.replace(/_[a-z-A-Z]+/, function(match) {
				return match.toUpperCase().replace('_', '-');
			});
		}
		return lang;
	};

	this.getLang = function(){
		return window.lang || 'en_us';
	};

	this.getInColumns = function(field, widget) {
		var inColumns = {};
		var inData = field.inData || {};

		Object.keys(inData).forEach(function(key) {
			inColumns[key] = widget.currentRow[inData[key]];
		});
		return inColumns;
	};

	this.buildFiltersInField = function(field, selectListSearch) {
		var filters = [];
		if (field) {
			if (Util.isArray(field.dataSourceFilter) && field.dataSourceFilter.length) {
				var dataSourceFilter = Util.clone(field.dataSourceFilter);
				dataSourceFilter.forEach(function(dataSource) {
					dataSource.value = self.evaluate(dataSource.value);
				});
				filters = filters.concat(dataSourceFilter);
			}

			if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
				selectListSearch = selectListSearch || "";
				field.filterDefinitions.fields.forEach(function(filterDefinition) {
					filterDefinition.value = selectListSearch + "%";
				});
				filters = filters.concat(field.filterDefinitions.fields);
			}

			var inColumns = this.getInColumns(field, field.widget);
			var query = Query.build();

			for (var column in inColumns) {
				query.where(column).equals(inColumns[column]);
			}

			filters = filters.concat(query.where());
		}
		return filters;
	};

	var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
	var ignoredPrefixes = ["__proxy_"];

	this.clearRow = function(row) {
		if(row) {
			var newRow = Util.clone(row, ignoredKeys);
			clearIgnoredPrefixes(newRow);
			delete newRow.$error;

			return newRow;
		}
		return row;
	};


	function clearIgnoredPrefixes(newRow) {
		Object.keys(newRow).forEach(function(key){
			ignoredPrefixes.forEach(function(prefix){
				if (key.indexOf(prefix) === 0) {
					delete newRow[key];
				}
			});
		});
	}

	this.orderBy = function(data, sortPredicate) {
		var orderByFilter = $filter('orderBy');
		return orderByFilter(data, Util.parseToArray(sortPredicate));
	};

	this.registerDataSource = function(name, localKey){
		localKey = Util.parseToArray(localKey);
		if(!dataSourceHolder.isLoaded(name)){
			var obj = {};
			obj[name] = {
				"localKey" : localKey
			};
			dataSourceHolder.setDataSource(name, obj);
		}
	};

	function isLikeBeginning(operator){
		return operator === "LIKE_BEGINNING";
	}

	function buildLikeAllQuery(widget, query, criteria){
		var columns = widget.fields
			.filter(fieldIsFilterable)
			.reduce(separatedFieldNamesReducer("|"), "");
		query = query.where(columns).likeAll(parseLikeCondition(criteria._ALL));
		return query;
	}

	function getInValues(field, criteria){
		var lowerCriteria = criteria.toLowerCase();
		var valueField = field.valueField;
		var descriptionField = field.descriptionField;
		var inValues = field.dataSource.data
			.filter(function(row){
				var lowerDescription = row[descriptionField].toLowerCase();
				return lowerDescription.indexOf(lowerCriteria) !== -1;
			}).map(function(row){
				return row[valueField];
			});

		return inValues;
	}

	function buildMappedLikeAllQuery(widget, query, criteria){
		var likeAllColumns = [];
		var mappedColumns = [];
		var mappedCriteria = {};
		widget.fields
			.filter(function(field){
				return fieldIsFilterable(field) && !Util.isUndefinedOrNull(field) && field.isVisible;
			})
			.forEach(function(field){
				if(field.dataSource && !field.dataSource.getStorageStrategy()){
					var values = getInValues(field, criteria._ALL_MAP);
					if(values.length){
						mappedColumns.push(field);
						mappedCriteria[field.name] = values;
					}
				} else {
					likeAllColumns.push(field);
				}
			});

		mappedColumns = mappedColumns.reduce(separatedFieldNamesReducer("&"), "");
		likeAllColumns = likeAllColumns.reduce(separatedFieldNamesReducer("|"), "");
		var columns = mappedColumns.length ? likeAllColumns + "#" + mappedColumns : likeAllColumns;

		query = query.where(columns).mappedLikeAll(parseMappedLikeConditions(criteria._ALL_MAP, mappedCriteria));
		return query;
	}

	/*
	 * Creates a filter from the widget's fields and a given criteria
	 * @param {object} widget the widget to be filtered
	 * @param {object} criteria the criteria to search for, that can contain the _ALL key (for every field) and other conditions for specific field names
	 * @returns {array} The filter array
	 */
	this.getLikeAllFilter = function getLikeAllFilter(widget, criteria) {
		var query = Query.build();
		Object.keys(criteria).forEach(function(key) {
			if (!Util.isEmptyOrBlank(criteria[key])) {
				if (key === "_ALL") {
					query = buildLikeAllQuery(widget, query, criteria);
				} else if(key === "_ALL_MAP") {
					query = buildMappedLikeAllQuery(widget, query, criteria);
				} else {
					var field = widget.getField(key);
					if (fieldIsFilterable(field)) {
						query = query.where(key).likeInsensitive( isLikeBeginning(field.operator) ? parseLikeBeginningCondition(criteria[key]) : parseLikeCondition(criteria[key]));
					}
				}
			}
		});
		return query.where();
	};

	/*
	 * Creates a filter for ordering a datasource
	 * @param {object} widget the widget to be ordered
	 * @param {object} orderColumns the columns to order
	 * @returns {array} The filter array
	 */
	this.getOrderByFilter = function getOrderByFilter(orderColumns) {
		var query = Query.build();
		var order = [];
		orderColumns.forEach(function(column) {
			var reverse = column[0] == '-';
			var direction = reverse ? Query.ORDER_DESC : Query.ORDER_ASC;
			var fieldName = reverse ? column.substr(1) : column;
			order.push({
				name: fieldName,
				direction: direction
			});
		});
		query.orderBy(order);
		return query.where();
	};

	function fieldIsFilterable(field) {
		return (
			!Util.isUndefinedOrNull(field) &&
			field.aclIsVisible &&
			(field.showOnList || field.showOnFilter) &&
			!field.skipSearch
		);
	}

	function mapFieldName(field) {
		return field.name;
	}

	function separatedFieldNamesReducer(separator) {
		return function (accumulatedValue, field) {
			return accumulatedValue + (accumulatedValue === "" ? "" :  separator) + field.name;
		};
	}

	function parseCondition(condition){
		if(isStrictCondition(condition)) {
			condition = condition.substring(1, condition.length - 1);
		} else {
			condition = condition.replace(/(\ +)/g, "%");
		}
		return condition;
	}

	function parseLikeCondition(condition){
		condition = parseCondition(condition);
		return "%" + condition + "%";
	}

	function parseMappedLikeConditions(likeAllCondition, mappedConditions){
		mappedConditions._ALL = parseLikeCondition(likeAllCondition);
		return [mappedConditions];
	}

	function parseLikeBeginningCondition(condition){
		condition = parseCondition(condition);
		return condition + "%";
	}

	function isStrictCondition(condition) {
		return Util.stringStartsWith(condition, "\"") && Util.stringEndsWith(condition, "\"");
	}

}